@@ -169,9 +169,37 @@ def get_compiled_model(inputs, meta_outputs, output_activation, num_predicts, nu
     )
     return model
 ###############################################################################
+# Reset Keras Session
+def reset_keras():
+    sess = get_session()
+    K.clear_session()
+    sess.close()
+    sess = get_session()
+
+    try:
+        del opt_model  ### delete this if it exists 
+        del best_model # this is from global space - change this as you need
+        del deep_model  ### delete this if it exists 
+        print('deleted deep and best models from memory')
+    except:
+        pass
+
+    print(gc.collect()) # if it does something you should see a number as output
+
+    # use the same config as you used to create the session
+    config = tf.compat.v1.ConfigProto()
+    config.gpu_options.per_process_gpu_memory_fraction = 1
+    config.gpu_options.visible_device_list = "0"
+    set_session(tf.compat.v1.Session(config=config))
+##################################################################################
 def build_model_optuna(trial, inputs, meta_outputs, output_activation, num_predicts, 
                         num_labels, optimizer_options, loss_fn, val_metrics, cols_len):
 
+    tf.compat.v1.reset_default_graph()
+    K.clear_session()
+    reset_keras()
+    tf.keras.backend.reset_uids()
+
     n_layers = trial.suggest_int("n_layers", 1, 4)
     num_hidden = trial.suggest_int("n_units", 32, 512, log=True)
     weight_decay = trial.suggest_float("weight_decay", 1e-8, 1e-3, log=True)
@@ -180,19 +208,19 @@ def build_model_optuna(trial, inputs, meta_outputs, output_activation, num_predi
     batch_norm = trial.suggest_categorical("batch_norm", [True, False])
     add_noise = trial.suggest_categorical("add_noise", [True, False])
     dropout = trial.suggest_float("dropout", 0, 0.5)
-
+    activation_fn = trial.suggest_categorical("activation", ['relu', 'tanh', 'elu', 'selu'])
     for i in range(n_layers):
         model.add(
             tf.keras.layers.Dense(
                 num_hidden,
                 name="opt_dense_"+str(i),
-                activation=trial.suggest_categorical("activation", ['relu', 'tanh', 'elu', 'selu']),
                 kernel_regularizer=tf.keras.regularizers.l2(weight_decay),
             )
         )
+        model.add(Activation(activation_fn,name="opt_activation_"+str(i)))
 
         if batch_norm:
-            model.add(BatchNormalization())
+            model.add(BatchNormalization(name="opt_batchnorm_"+str(i)))
 
         if add_noise:
             model.add(GaussianNoise(trial.suggest_float("adam_learning_rate", 1e-5, 1e-1, log=True)))
@@ -439,6 +467,13 @@ def return_optimizer(hp, hpq_optimizer, trial_flag=False):
         else:
             best_optimizer = nesterov
     return best_optimizer
+##########################################################################################
+from tensorflow.keras import backend as K
+import tensorflow
+import gc
+from tensorflow.python.keras.backend import get_session, set_session
+import tensorflow as tf
+
 ##########################################################################################
 import optuna
 def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type, 
@@ -482,6 +517,7 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     #optimizer = SGD(lr=learning_rate, momentum = 0.9)
     print('    Using optimizer = %s' %str(optimizer).split(".")[-1][:8])
     use_bias = check_keras_options(keras_options, 'use_bias', True)
+    lr_scheduler = check_keras_options(keras_options, 'lr_scheduler', "")
     #######################################################################
     val_mode = keras_options["mode"]
     val_monitor = keras_options["monitor"]
@@ -541,6 +577,10 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     else:
         tuner = model_options["tuner"]
     print('Training %s model using %s. This will take time...' %(keras_model_type, tuner))
+
+    from secrets import randbelow
+    rand_num = randbelow(10000)
+    tf.compat.v1.reset_default_graph()
     K.clear_session()
     ############################################################################
     ########     P E R FO R M     T U N I N G    H E R E  #####################
@@ -624,12 +664,22 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
         deep_model = get_compiled_model(inputs, deep_outputs, output_activation, num_predicts, 
                             num_labels, deep_model, best_optimizer, val_loss, val_metrics, cols_len)
     elif tuner.lower() == "optuna":
+        # Reset Keras Session
         ######     O P T U N A    ###########################
         ### This is where you build the optuna model #####
         ####################################################
         optuna_scores = []
         def objective(trial):
             optimizer_options = ""
+            tf.compat.v1.reset_default_graph()
+            K.clear_session()
+            reset_keras()
+            tf.keras.backend.reset_uids()
+            try:
+                del opt_model
+                print('deleted opt model from memory')
+            except:
+                pass
             opt_model = build_model_optuna(trial, inputs, meta_outputs, output_activation, num_predicts, 
                         num_labels, optimizer_options, val_loss, val_metrics, cols_len)
             cb_default = callbacks.LearningRateScheduler(lambda epoch: learning_rate * (0.75 ** np.floor(epoch / 2)))
@@ -651,10 +701,11 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
             optuna_scores.append(score)
             return score
         ##### This where you run optuna ###################
+        study_name = project_name+'_'+keras_model_type+'_study_'+str(rand_num)
         if val_mode == 'max':
-            study = optuna.create_study(direction="maximize")
+            study = optuna.create_study(study_name=study_name, direction="maximize", load_if_exists=False)
         else:
-            study = optuna.create_study(direction="maximize")
+            study = optuna.create_study(study_name=study_name, direction="minimize", load_if_exists=False)
         ### now find the best tuning hyper params here ####
         study.optimize(objective, n_trials=max_trials)
         print('Best trial score in Optuna: %s' %study.best_trial.value)
@@ -712,6 +763,7 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     onecycle_steps = max(10, np.ceil(data_size/(2*batch_size))*NUMBER_OF_EPOCHS)
     print('Onecycle steps = %d' %onecycle_steps)
     onecycle = OneCycleScheduler(steps=onecycle_steps, lr_max=0.05)
+
     if keras_options['lr_scheduler'] == "lr_scheduler":
         lr_scheduler = lr_sched
         keras_options['lr_scheduler'] = "lr_sched"
@@ -720,7 +772,7 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     elif keras_options['lr_scheduler'] == 'rlr':
         lr_scheduler = rlr   #onecycle
     elif keras_options['lr_scheduler'] == 'decay':
-        lr_scheduler == lr_decay_cb
+        lr_scheduler = lr_decay_cb
     else:
         lr_scheduler = onecycle
         keras_options['lr_scheduler'] = "onecycle"
@@ -758,13 +810,23 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     print('Model training with best hyperparameters for %d epochs' %NUMBER_OF_EPOCHS)
     for each_callback in callbacks_list:
         print('    Callback added: %s' %str(each_callback).split(".")[-1])
+    
     ####  Do this with LR scheduling but NO early stopping since we want the model fully trained #####
     history = best_model.fit(train_ds, validation_data=valid_ds, batch_size=best_batch,
                 epochs=NUMBER_OF_EPOCHS, steps_per_epoch=STEPS_PER_EPOCH, 
                 callbacks=callbacks_list, 
                 validation_steps=STEPS_PER_EPOCH,
                shuffle=False)
+
+    #################################################################################
+    #######          R E S E T    K E R A S      S E S S I O N 
+    #################################################################################
+    # Reset Keras Session
+
     K.clear_session()
+    reset_keras()
+    tf.compat.v1.reset_default_graph()
+    tf.keras.backend.reset_uids()
     
     ###   Once the best learning rate is chosen the model is ready to be trained on full data
     print('    Model training metrics available: %s' %history.history.keys())
@@ -848,6 +910,10 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     print('         Held-out test data set Results:')
     num_labels = cat_vocab_dict['num_labels']
     num_classes = cat_vocab_dict['num_classes']
+    if check_for_nan_in_array(y_probas):
+        y_probas = pd.DataFrame(y_probas).fillna(0).values
+    elif check_for_nan_in_array(y_test_preds):
+        y_test_preds = pd.DataFrame(y_test_preds).fillna(0).values.ravel()
     if num_labels <= 1:
         if modeltype == 'Regression':
             print_regression_model_stats(y_test, y_test_preds,target,plot_name="deep_autoviml")
@@ -916,8 +982,8 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     else:
         pass
         print('    best learning rate = %s' %best_rate)
-    #K.set_value(deep_model.optimizer.learning_rate, best_rate)
-    #print("    set learning rate using best model:", deep_model.optimizer.learning_rate.numpy())
+    K.set_value(deep_model.optimizer.learning_rate, best_rate)
+    print("    set learning rate using best model:", deep_model.optimizer.learning_rate.numpy())
     ####   Dont set the epochs too low - let them be back to where they were stopped  ####
     print('    max epochs for training = %d' %stopped_epoch)
     deep_model.fit(full_ds, epochs=stopped_epoch, steps_per_epoch=STEPS_PER_EPOCH, 
@@ -947,6 +1013,18 @@ def train_custom_model(inputs, meta_outputs, full_ds, target, keras_model_type,
     else:
         print('\nModel not being saved since save_model_flag set to False...')
 
+    #############################################################################
+    #####     C L E A R      S E S S I O N     B E F O R E   C L O S I N G   ####
+    #############################################################################
+    #from numba import cuda
+    #cuda.select_device(0)
+    #cuda.close()
+    # Reset Keras Session
+    K.clear_session()
+    tf.compat.v1.reset_default_graph()
+    reset_keras()
+    tf.keras.backend.reset_uids()
+
     #### make sure you save the cat_vocab_dict to use later during predictions
     try:
         pickle_path = os.path.join(project_name,"cat_vocab_dict")+".pickle"
@@ -999,3 +1077,11 @@ def return_model_body(keras_options):
                                     activity_regularizer=tf.keras.regularizers.l2(0.01)))
     return model_body
 ########################################################################################
+def check_for_nan_in_array(array_in):
+    """
+    If an array has NaN in it, this will return True. Otherwise, it will return False.
+    """
+    array_sum = np.sum(array_in)
+    array_nan = np.isnan(array_sum)
+    return array_nan
+########################################################################################
\ No newline at end of file
